package com.doc.common.utils;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.text.PDFTextStripper;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.extractor.WordExtractor;
import org.apache.poi.xslf.usermodel.XMLSlideShow;
import org.apache.poi.xslf.usermodel.XSLFShape;
import org.apache.poi.xslf.usermodel.XSLFSlide;
import org.apache.poi.xslf.usermodel.XSLFTextShape;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * 读取文件内容
 * WPS WORD TXT
 *
 * @author wukai
 */
public class FileContentUtils {

    /**
     * 获取正文文件内容，docx方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentDocx(byte[] data) {
        // 0表示获取正常，1表示获取异常
        try (InputStream is = new ByteArrayInputStream(data)) {
            //根据需求入参也可以改为文件路径，对应的输入流部分改为new File(路径)即可
            // 2007版本的word
            XWPFDocument xwpf = new XWPFDocument(is);
            // 2007版本，仅支持docx文件处理
            List<XWPFParagraph> paragraphs = xwpf.getParagraphs();
            StringBuilder content = new StringBuilder();

            for (XWPFParagraph paragraph : paragraphs) {
                if (!paragraph.getParagraphText().startsWith("    ")) {
                    content.append(paragraph.getParagraphText().trim()).append("\r\n");
                } else {
                    content.append(paragraph.getParagraphText());
                }
            }
            return content.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，doc方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentDoc(byte[] data) {
        try (InputStream is = new ByteArrayInputStream(data)) {
            // 2003版本的word
            WordExtractor wordExtractor = new WordExtractor(is);
            // 2003版本 仅doc格式文件可处理，docx文件不可处理
            String[] text = wordExtractor.getParagraphText();
            StringBuilder content = new StringBuilder();

            // 获取段落，段落缩进无法获取，可以在前添加空格填充
            for (String str : text) {
                if (!str.startsWith("    ")) {
                    content.append(str.trim()).append("\r\n");
                } else {
                    content.append(str);
                }
            }
            return content.toString();
        } catch (IllegalArgumentException e) {
            //做兼容,因为onlyoffice在线编辑时，会将2003的文档保存为word2007的格式。
            return getContentDocx(data);
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，wps方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentWps(byte[] data) {
        // 0表示获取正常，1表示获取异常
        try (InputStream is = new ByteArrayInputStream(data)) {
            // wps版本word
            HWPFDocument hwpf = new HWPFDocument(is);

            // 文档文本内容
            String[] text = new WordExtractor(hwpf).getParagraphText();
            StringBuilder content = new StringBuilder();
            if (text != null && text.length > 0) {
                for (String str : text) {
                    if (!str.startsWith("    ")) {
                        content.append(str.trim()).append("\r\n");
                    } else {
                        content.append(str);
                    }
                }
            }
            return content.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，PDF方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentPdf(byte[] data) {
        String result = "";
        try {
            PDDocument load = PDDocument.load(data);
            PDFTextStripper stripper = new PDFTextStripper();
            stripper.setSortByPosition(true);
            for (int i = 1; i < load.getNumberOfPages() + 1; i++) {
                //读取pdf的开始到结束页
                stripper.setStartPage(i);
                stripper.setEndPage(i);
                String text = stripper.getText(load);
                //拼接不同页数的数据返回
                result += text;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取正文文件内容，PDF方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentPpt(byte[] data) {
        String result = "";
        try (InputStream is = new ByteArrayInputStream(data)) {
            XMLSlideShow ppt = new XMLSlideShow(is);
            List<XSLFSlide> slideList = ppt.getSlides();
            for (XSLFSlide slide : slideList) {
                List<XSLFShape> shapeList = slide.getShapes();
                for (XSLFShape shape : shapeList) {
                    if (shape instanceof XSLFTextShape) {
                        XSLFTextShape textSharp = (XSLFTextShape) shape;
                        result += textSharp.getText() + "\n";
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result.replace("\n\n", "\n");
    }

    public static String getContentTxt(byte[] data) {
        return new String(data, StandardCharsets.UTF_8).trim();
//        try (InputStream is = new ByteArrayInputStream(data); BufferedReader br = new BufferedReader(data)) {
//            StringBuilder sb = new StringBuilder();
//            String line;
//            while ((line = br.readLine()) != null) {
//                sb.append(line).append(System.lineSeparator());
//            }
//            return sb.toString();
//        } catch (IOException e) {
//            e.printStackTrace();
//            return "";
//        }
    }

    public static void main(String[] args) {
    }
}
